MODULE MAP_Steel_Free_Energy
   PRIVATE
   PUBLIC :: MAP_STEEL_FTO1,MAP_STEEL_G91,AJ
   CONTAINS

! -------------------------------------------------------------------
!
! The procedures in this module calculate the free energy change 
!  accompanying the transformation from austenite to ferrite of the same
!  chemical composition, and also the differential of that free energy
!  change.
!
! MODULE contains:
!
!   MAP_STEEL_FTO1
!       Calculates free energy change.
!   MAP_STEEL_G91
!       Calculates differential of free energy change.
!
!   In  all cases, the parameters accepted by the procedures are:
!
!  H: excess partial molar enthalpy of solution of carbon in 
!      austenite (J/mol).
!  S: excess partial molar enthalpy of solution of carbon in 
!      ferrite (J/mol).
!  X: mole fraction of carbon in ferrite at the T0 boundary.
!  T: absolute temperature.
!  W: carbon-carbon interaction energy in austenite (J/mol).
!  W1:carbon-carbon interaction energy in ferrite (J/mol).
!  F: free energy change for transformation of austenite to ferrite of the 
!      same chemical composition (J).
!
!   Procedures in this module also use Module Global_Data.
!
!   This Module also contains :
!
!     ZEN_CALC
!        Calculates Zener ordering term, based on T and X.
!
!     AJ
!        Calculates AJ = 1-e(W/RT), where R is the gas constant.
!
! -----------------------------------------------------------------------
!
! Function MAP_STEEL_FTO1
!
! Produced by :-
!
! H. K. D. H. Bhadeshia
!
! Department of Materials Science and Metallurgy,
! Pembroke Street, Cambridge CB2 3QZ, U. K.
! 31 March 1987
!
! Calculates the free energy change accompanying the transformation
! from austenite to ferrite of the same chemical composition, including
! a Zener ordering term. The latter describes the ordering of carbon
! atoms, that leads the body-centered cubic lattice of ferrite becoming
! body-centered tetragonal. The degree of ordering increases as the
! carbon concentration increases, or as the transformation temperature
! decreases.
!
! References:
!
!  1. C. Zener Trans. AIME 167 (1946) 550.
!
!  2. J. C. Fisher, Metals Transactions 185 (1949) 688-690.
!
!  3. H. K. D. H. Bhadeshia and D. V. Edmonds
!      Acta Metallurgica 28 (1980) 1265-1273.
!      Also see laboratory book 7.
!
! ----------------------------------------------------------------------
       FUNCTION MAP_STEEL_FTO1(F,H,S,T,W,W1,X)

       USE MAP_Util_Global_Data
       IMPLICIT NONE
! ----------------------------------------------------------------------
!
! Arguments.
!
! ----------------------------------------------------------------------
       REAL(KIND=real_8_30) ::  MAP_STEEL_FTO1
       REAL(KIND=real_8_30),INTENT(IN)  :: F,H,S,T,W,W1,X
! ----------------------------------------------------------------------
!
! Local variables.
!
! ----------------------------------------------------------------------

       REAL(KIND=real_8_30) :: D,D11,ZENER,ZEN1,ZEN2

!
! Calculate the free
!  energy change accompanying the change from austenite to ferrite of the
!  same composition, including Zener ordering term (using sub ZEN_CALC).
! See Bhadeshia and Edmonds [3].
!
       D  = SQRT(1-2*(1+2*AJ(T,W))*X+(1+8*AJ(T,W))*X**2)
       D11= SQRT(9-6*X*(2*AJ(T,W1)+3)+(9+16*AJ(T,W1))*X**2)

       CALL ZEN_CALC(T,X,ZENER,ZEN1,ZEN2)

       MAP_STEEL_FTO1 = X*R*T*LOG(X**2)+						 &
     &                  X*(H_T(T)-(H)-(S_T(T)-(S))*T+4*W1-6*W)-  &
     &                  R*T*(1-X)*LOG((1-X)**4)+				 &
     &                  5*R*T*(1-2*X)*LOG(1-2*X)-				 &
     &                  R*T*X*LOG(((D-1+3*X)/(D+1-3*X))**6)-	 &
     &                  R*T*(1-X)*LOG((                          &
     &				(1-2*AJ(T,W)+(4*AJ(T,W)-1)*X-D)/				 &
     &				(2*AJ(T,W)*(2*X-1))						 &
     &                    )**6									 &
     &                  )+                                       &
     &                  3*R*T*X*LOG(3-4*X)+		     			 &
     &                  R*T*X*LOG(((D11-3+5*X)/(D11+3-5*X))**4)+ &
     &                  (1-X)*F+                                 &
     &                  ZENER

       RETURN
	   END FUNCTION MAP_STEEL_FTO1
! -----------------------------------------------------------------------
! 
! Function MAP_STEEL_G91
!
! Produced by :-
!
! H.K.D.Bhadeshia,
! Phase Transformations Group,
! Department of Materials Science and Metallurgy,
! University of Cambridge,
! Cambridge, UK.
!
! Purpose :-
!
! To calculate the differential of MAP_STEEL_FTO1 with respect to XTO
!  (see subroutine MAP_STEEL_FTO1 for description of parameters).  The 
!  differentiation is used in order to apply the Newton Iteration.
!
! Unpublished work, H. K. D. H. Bhadeshia, University of Cambridge
!  See laboratory book 5B
!
! Input parameters :-
!
! XTO is the carbon concentration at the T0 boundary.
! T is the temperature (in kelvin).
! W is the carbon-carbon interaction energy in austenite 
!  (in joules per mole).
! W1 is the carbon-carbon interaction energy in ferrite 
!  (in joules per mole).
! H_T(T) is the excess partial molar enthalpy of solution of carbon
!  in ferrite (in joules per mole).
! S_T(T) is the excess partial molar entropy of solution of carbon
!  in ferrite (in joules per mole per kelvin).
! F is the energy.
! H is the excess partial molar enthalpy of solution of carbon
!  in austenite (in joules per mole).
! S is the excess partial molar entropy of solution of carbon
!  in austenite (in joules per mole).
! AJ  = 1 - e(W/RT), where R is the universal gas constant.
! AJ1 = 1 - e(W1/RT), where R is the universal gas constant.
! R is the universal gas constant (in joules per mole per kelvin)
!
! ----------------------------------------------------------------------
FUNCTION MAP_STEEL_G91(F,H,S,T,W,W1,XTO)
  USE MAP_Util_Global_Data
  IMPLICIT NONE
!
! Input arguments.
!
  REAL(KIND=real_8_30),INTENT(IN) :: F,H,S,T,W,W1,XTO
  REAL(KIND=real_8_30) MAP_STEEL_G91
!
! Local variables.
!
  REAL(KIND=real_8_30) :: DT5,DT6,FD,FD1,G1, &
    DZEN1,DZEN2,DZEN6,DZEN7,DZEN8, &
    V1,V2,V3,V4,V5,V6,V7,V8,V9,ZENER

  DT5 = 2.808E+04*XTO/(1-XTO)
  DT6 = T/DT5

  FD = SQRT(1-2*(1+2*AJ(T,W))*XTO+(1+8*AJ(T,W))*XTO**2)
  FD1= SQRT(9-6*XTO*(2*AJ(T,W1)+3)+(9+16*AJ(T,W1))*XTO**2)

  V1 = FD-1+3*XTO
  V2 = FD+1-3*XTO
  V3 = 1-2*AJ(T,W)+(4*AJ(T,W)-1)*XTO-FD
  V4 = 2*AJ(T,W)*(2*XTO-1)
  V5 = FD1-3+5*XTO
  V6 = FD1+3-5*XTO
  V7 = (XTO-1-2*AJ(T,W)+8*XTO*AJ(T,W))/FD
  V8 = (9*XTO-9-6*AJ(T,W1)+16*AJ(T,W1)*XTO)/FD1
  V9 = H_T(T)-(H)-(S_T(T)-(S))*T-6*W+4*W1

  G1 = 2 	               &
       +LOG(XTO**2)        &
    +4 +LOG((1-XTO)**4)    &
   -10 -LOG((1-2*XTO)**10) &
       -LOG((V1/V2)**6)    & 
    -6 *((XTO/V1)*(V7+3+V1*(3-V7)/V2))                         &
       +LOG((V3/V4)**6) -6 *((1-XTO)/V3)*(4*AJ(T,W)*(1-(V3/V4))-V7) &
    +3*(LOG(3-4*XTO) -4*XTO/(3-4*XTO)) &
       +LOG((V5/V6)**4)                &
    +4*(XTO/V5) *(V8+5+(V5/V6)*(5-V8))

  CALL ZEN_CALC(T,XTO,ZENER,DZEN1,DZEN2)

  IF (DT6 .GT. 1.0) THEN
     DZEN8 = 0.0
  ELSE
     DZEN6 = (-3.3948 +13.6112 *DT6 -13.4376 *(DT6**2)) &
 &            *T/(28080*((1-XTO)**2))
     DZEN7 = (-3.3118 +15.7462 *DT6 -23.2449 *(DT6**2)) &
 &            *T/(28080*((1-XTO)**2))
     DZEN8 = 50898 *((DZEN2*XTO)**2)/((1-XTO)**2)			     &
 &            +(-50898* (2* DZEN2 *DZEN6 *(XTO**2) +2 *XTO	     &
 &            *(DZEN2**2))/(1-XTO)) +DZEN1 *T *0.6623741 +DZEN7 &
 &            *T *XTO *0.6623741
  END  IF

  MAP_STEEL_G91 = V9 +R *T *G1 +DZEN8 -F

  RETURN
END FUNCTION MAP_STEEL_G91
! -----------------------------------------------------------------------
!
!  Subroutine ZEN_CALC
!
!    Calculates effect of Zener ordering (if any).  Called from 
!    functions MAP_STEEL_FTO1 (needs ZENER) and MAP_STEEL_G91 
!    (needs ZEN1 and ZEN2).
!
! -----------------------------------------------------------------------
  SUBROUTINE ZEN_CALC(T,X,ZENER,ZEN1,ZEN2)
  USE MAP_Util_Global_Data
  IMPLICIT NONE
!
! Arguments
!
  REAL(KIND=real_8_30),INTENT(IN)  :: T,X
  REAL(KIND=real_8_30),INTENT(OUT) :: ZENER,ZEN1,ZEN2
!
!  Local Variables
!
  REAL(KIND=real_8_30) :: T60,ZENTOT

!
! Calculate T60, which is the value of the actual temperature T
!   divided by the critical temperature beyond which there is no
!   Zener ordering. X is the mole fraction of carbon in the ferrite.
! If T60>1, the transformation occurs above the ordering temperature
!   and Zener ordering does not occur.
!

    T60 = T*(1-X)/(2.808E+04*X)

    IF (T60 .GT. 1.0E+00) THEN
          ZENER=0.0
    ELSE
!
! Calculate the contribution to the free energy change due to
!  Zener ordering.
!
          IF (T60 .GT. 2.5E-01) THEN
             ZEN2 = 1.0E+00
             ZEN1 = 3.295E+00
          ELSE
!
! Polynomial fit to data in Table 2 of Fischer (1949)
!
          ZEN1=  0.2307          &
		       +42.7974* T60     &
			  -233.8631*(T60**2) &
			  +645.4485*(T60**3) &
              -954.3995*(T60**4) &
		      +711.8095*(T60**5) &
		      -211.5136*(T60**6)

          ZEN2= -2.6702	         &
		       +45.6337* T60     &
		      -225.3965*(T60**2) &
		      +567.7112*(T60**3) &
              -771.6466*(T60**4) &
		      +538.1778*(T60**5) &
              -151.3818*(T60**6)
          END  IF

          ZENTOT=(((ZEN2*X)**2)*(-50898.56)/(1-X))+ZEN1*T*X*0.6623741
!
! Convert from calories per mole to Joules per mole
!
          ZENER=ZENTOT*4.187E+00

    END  IF


  RETURN
  END SUBROUTINE ZEN_CALC
! -----------------------------------------------------------------------
END MODULE MAP_Steel_Free_Energy
